/****************************************************************************/
/*                                                                          */
/*  lock_process.c  ;user interface to lock_process.icb daemon              */
/*                  ;variable-length hog_processor() from 0 to 2^32-1 msec  */
/*                                                                          */
/*  by                                                                      */
/*                                                                          */
/*  Dr. Richard F. Drushel                                                  */
/*  Department of Biology                                                   */
/*  Case Western Reserve University                                         */
/*  10900 Euclid Avenue                                                     */
/*  Cleveland, Ohio  44106-7080  USA                                        */
/*  rfd@po.cwru.edu                                                         */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*  VERSION HISTORY:                                                        */
/*                                                                          */
/*  Whenever the program is updated, record it here.  Add new version info  */
/*  to the top of this list, so the newest version is always first.         */
/*                                                                          */
/*  1.0      03 Mar 2000         ;RFD initial coding                        */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*  PUBLIC FUNCTIONS:                                                       */
/*                                                                          */
/*  These functions may be used freely by user programs.                    */
/*                                                                          */
/*  void lock_process(float time)       ;hog_processor() for time seconds   */
/*                                                                          */
/*  void unlock_process()               ;releases lock_process() lock       */
/*                                      ;and defer() to next process        */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*  PUBLIC GLOBAL VARIABLES:                                                */
/*                                                                          */
/*  These global variables may be accessed freely by user programs.  Any    */
/*  restrictions on their use (e.g., flags which are read-only) should be   */
/*  noted here.  It is suggested that global variables be named in all      */
/*  capital letters, to avoid confusion with local varibles which are       */
/*  defined within functions.                                               */
/*                                                                          */
/*  None.                                                                   */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*  PRIVATE FUNCTIONS:                                                      */
/*                                                                          */
/*  These are internal functions which have no public entry points, and     */
/*  which user programs should not access directly.  It is suggested that   */
/*  private functions be named with a leading underscore (_), to avoid      */
/*  confusion with user-defined functions which might have the same name.   */
/*                                                                          */
/*  None.                                                                   */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*  PRIVATE GLOBAL VARIABLES:                                               */
/*                                                                          */
/*  These are internal global variables which user programs should not      */
/*  access directly.  It is suggested that private global variables be      */
/*  named in all capital letters with a leading underscore (_), to avoid    */
/*  confusion with user-defined global variables which might have the same  */
/*  name.                                                                   */
/*                                                                          */
/*  NOTE:  these are all declared in lock_process.icb!!!                    */
/*                                                                          */
/*  int _LOCK_PROCESS_FLAG          ;signals lock_process() daemon that     */
/*                                  ;the current process is locked in.      */
/*                                  ;0=not locked, non-zero=locked          */
/*                                                                          */
/*  int _LOCK_PROCESS_TICK_COUNT_HI ;hiword of lock tick counter (msec)     */
/*                                                                          */
/*  int _LOCK_PROCESS_TICK_COUNT_LO ;loword of lock tick counter (msec)     */
/*                                                                          */
/****************************************************************************/
/*                                                                          */
/*  EXTERNAL LIBRARY FILE DEPENDENCIES:                                     */
/*                                                                          */
/*  Any external functions or variables which are used but not defined in   */
/*  this file should be noted here.  List the external library filename,    */
/*  the library function, and the local calling function.                   */
/*                                                                          */
/*  lock_process.icb                ;must be loaded first to install        */
/*                                  ;the daemon                             */
/*                                                                          */
/*  lib_????.c                      ;hog_processor() used by                */
/*                                  ;lock_process() and unlock_process()    */
/*                                                                          */
/*  math.c                          ;int2uf() used by lock_process()        */
/*                                                                          */
/****************************************************************************/


/****************************************************************************/
/*                                                                          */
/*                        global variable declarations                      */
/*                                                                          */
/****************************************************************************/

/* declare global variables here */

/* None declared here, but 3 declared in lock_process.icb. */

/****************************************************************************/
/*                                                                          */
/*                            function declarations                         */
/*                                                                          */
/****************************************************************************/

/* declare functions here */

void lock_process(float time)       /* hog_processor() for time seconds */
                                    /* overhead is about 4-6 msec       */
{
    float loword,hiword,t;

    hog_processor();                /* grab the CPU for 256 msec */

    if (time<=0.)
        defer();                    /* abort with nonsensical arguments */
    else
    {
        _LOCK_PROCESS_FLAG=0;           /* make sure it's off so that we can */
                                        /* write to the counter without fear */
                                        /* that the daemon will interrupt us */
                                        /* and decrement it                  */

        t=time*1000.;                   /* convert seconds to msec */
        if (t>4294967000.)              /* truncate if too big */
            t=4294967000.;              /* (almost) max msec in 32-bit counter  */
                                        /* some fudge because of roundoff error */

        hiword=int2uf((int)(t/65536.)); /* make sure it's *unsigned* */
        loword=t-(hiword*65536.);
    
        _LOCK_PROCESS_TICK_COUNT_HI=(int)(hiword);  /* initialize the counters */
        _LOCK_PROCESS_TICK_COUNT_LO=(int)(loword);

        _LOCK_PROCESS_FLAG=1;           /* turn on the lock, signalling the */
                                        /* daemon to start decrementing the */
                                        /* counters at each timer tick      */
    }
}

/****************************************************************************/

void unlock_process()           /* release the lock_process() lock */
{
    hog_processor();            /* grab the CPU for 256 msec           */
                                /* we do this just in case someone is  */
                                /* calling this without having called  */
                                /* lock_process() first                */

    _LOCK_PROCESS_FLAG=0;       /* clear the flag, signalling the daemon */
                                /* to stop decrementing the counter      */

    _LOCK_PROCESS_TICK_COUNT_HI=0;      /* zero the counters for next time */
    _LOCK_PROCESS_TICK_COUNT_LO=0;

    defer();                    /* and swap to the next process */

}

/****************************************************************************/

